import { type Repository } from '@slovo/domain/repository';
import { Entity, type EntityId } from '@slovo/domain/entity';
import { type User } from '@slovo/models/user';
import { type PoolConnection } from 'mariadb';
import connectionPool from '@slovo/infrastructure/database-pool';

export class UserRepository implements Repository<User> {
  public async get(id: EntityId): Promise<Entity<User>> {
    const conn: PoolConnection = await connectionPool.getConnection();

    const query = `SELECT * FROM users WHERE id = ?`;
    const rows = await conn.query(query, [ id ]);

    if (!rows.length) {
      throw Error("Unable to find User");
    }

    const row = rows.shift();

    const user: User = {
      name: row.name,
      email: row.email,
      password: row.password
    };

    await conn.release();
    return new Entity(id, user);
  }

  public async save(user: User): Promise<Entity<User>> {
    const conn: PoolConnection = await connectionPool.getConnection();

    const query = `INSERT INTO users (name, email, password) VALUES (?, ?, ?)`;
    const rows = await conn.query(query, [ user.name, user.email, user.password ]);

    if (!rows.length) {
      throw Error("Unable to create User");
    }

    await conn.release();
    return new Entity(rows.insertId, user);
  }

  public async update(user: Entity<User>): Promise<Entity<User>> {
    const conn: PoolConnection = await connectionPool.getConnection();
    
    const query = `UPDATE user SET name = ?, email = ?, password = ? WHERE id = ?`;
    const results = await conn.query(query, [
      user.getModel().name,
      user.getModel().email,
      user.getModel().password,
      user.getId()
    ]);

    await conn.release();
  }

  public async delete(user: Entity<User>): Promise<boolean> {
    const conn: PoolConnection = await connectionPool.getConnection();

    await conn.release();
  }
}
